/*
 * FILTER.C
 *
 * Context filters
 */

#include <stdio.h>
#include <sys/file.h>
#include "dmail.h"

Prototype filter_t get_filter(int artno, const char **);

static void filter_text(const char *base, int bytes);
static void filter_quoted_printable(const char *base, int bytes);

filter_t
get_filter(int artno, const char **filter_namep)
{
	const char *encoding;
	int len;

	encoding = get_field("Content-Transfer-Encoding:", &len);
	*filter_namep = "text";
	if (encoding == NULL)
		return(filter_text);
	if (strncmp(encoding, "quoted-printable", 16) == 0) {
		*filter_namep = "quoted-printable";
		return(filter_quoted_printable);
	}
	return(filter_text);
}

static void
filter_text(const char *base, int bytes)
{
	WPAGER(base, bytes);
}

static void
filter_quoted_printable(const char *base, int bytes)
{
	int b;
	int i;
	char buf[32];

	for (b = i = 0; i < bytes; ++i) {
		if (base[i] == '=' && i + 2 < bytes) {
			if (i != b)
				WPAGER(base + b, i - b);
			if (isalnum(base[i+1])) {
				buf[0] = base[i+1];
				buf[1] = base[i+2];
				buf[2] = 0;
				buf[0] = strtol(buf, NULL, 16);
				buf[1] = 0;
				WPAGER(buf, 1);
				i = i + 2;
			} else {
				/* skip char */
				++i;
			}
			b = i + 1;
		}
	}
	if (bytes - b)
		WPAGER(base + b, bytes - b);
}

